@@ -180,15 +180,25 @@ class RWKV(MyModule):
                         else:
                             w[x] = w[x].float()
 
-                            w[x+'_mx'] = torch.amin(w[x], dim=0) * 0.7
-                            w[x] = w[x] - w[x+'_mx']
-                            w[x+'_my'] = torch.amin(w[x], dim=1).unsqueeze(1)
-                            w[x] = w[x] - w[x+'_my']
-                            w[x+'_rx'] = torch.amax(w[x], dim=0)
-                            w[x] = w[x] / w[x+'_rx']
-                            w[x+'_ry'] = torch.amax(w[x], dim=1).unsqueeze(1)
-                            w[x] = w[x] / w[x+'_ry']
-                            
+                            if w[x].shape[0] > w[x].shape[1]:
+                                w[x+'_my'] = torch.amin(w[x], dim=1).unsqueeze(1)
+                                w[x] = w[x] - w[x+'_my']
+                                w[x+'_mx'] = torch.amin(w[x], dim=0)
+                                w[x] = w[x] - w[x+'_mx']
+                                w[x+'_rx'] = torch.amax(w[x], dim=0)
+                                w[x] = w[x] / w[x+'_rx']
+                                w[x+'_ry'] = torch.amax(w[x], dim=1).unsqueeze(1)
+                                w[x] = w[x] / w[x+'_ry']
+                            else:
+                                w[x+'_mx'] = torch.amin(w[x], dim=0)
+                                w[x] = w[x] - w[x+'_mx']
+                                w[x+'_my'] = torch.amin(w[x], dim=1).unsqueeze(1)
+                                w[x] = w[x] - w[x+'_my']
+                                w[x+'_rx'] = torch.amax(w[x], dim=0)
+                                w[x] = w[x] / w[x+'_rx']
+                                w[x+'_ry'] = torch.amax(w[x], dim=1).unsqueeze(1)
+                                w[x] = w[x] / w[x+'_ry']
+
                             w[x] = torch.clip(torch.floor(w[x] * 256), min=0, max=255).to(dtype=torch.uint8)
                             w[x+'_mx'] = w[x+'_mx'].to(dtype=ATYPE)
                             w[x+'_rx'] = (w[x+'_rx'] / 16).to(dtype=ATYPE)
@@ -198,21 +208,21 @@ class RWKV(MyModule):
                         w[x] = w[x].to(dtype=ATYPE)
                 
                 if 'emb.' in x:
-                    pass
+                    w[x] = w[x].contiguous()
                 elif (dd.stream) and (x.endswith('key.weight') or x.endswith('value.weight') or x.endswith('receptance.weight') or x.endswith('output.weight')):
                     try:
-                        w[x] = w[x].pin_memory() # if you see "CUDA error: out of memory" here, that's out of CPU RAM, not VRAM. Get more RAM :)
+                        w[x] = w[x].contiguous().pin_memory() # if you see "CUDA error: out of memory" here, that's out of CPU RAM, not VRAM. Get more RAM :)
                     except:
                         print('Note: You are running out of RAM. Get more CPU RAM. Now this will run much slower.')
                 elif DEVICE != 'cpu':
-                    w[x] = w[x].to(device=DEVICE)
+                    w[x] = w[x].to(device=DEVICE).contiguous()
                 
                 if (dd.stream) or (DEVICE != 'cpu'):
                     try:
-                        w[x+'_mx'] = w[x+'_mx'].to(device=DEVICE)
-                        w[x+'_rx'] = w[x+'_rx'].to(device=DEVICE)
-                        w[x+'_my'] = w[x+'_my'].to(device=DEVICE)
-                        w[x+'_ry'] = w[x+'_ry'].to(device=DEVICE)
+                        w[x+'_mx'] = w[x+'_mx'].to(device=DEVICE).contiguous()
+                        w[x+'_rx'] = w[x+'_rx'].to(device=DEVICE).contiguous()
+                        w[x+'_my'] = w[x+'_my'].to(device=DEVICE).contiguous()
+                        w[x+'_ry'] = w[x+'_ry'].to(device=DEVICE).contiguous()
                     except:
                         pass
 
